/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#include <iostream>
#include "Attributes.h"
#include "Inventory.h"
#include "Builder.h"
#include "Builder.h"
#include "Kind.h"
#include "Style.h"
#include "Type.h"
#include "Wood.h"

using namespace std;

/**
 * Initialize an inventory with instruments.
 * @param inventory the inventory to initialize.
 */
void initialize_inventory(Inventory *inventory);

/**
 * The main.
 */
int main()
{
    Inventory *inventory = new Inventory();
    initialize_inventory(inventory);

    // Create a search target with attributes to match.
    Attributes *attributes = new Attributes();
    attributes->set(Key::BUILDER,   Builder::GIBSON);
    attributes->set(Key::BACK_WOOD, Wood::MAPLE);
    InstrumentSpec *target_spec = new InstrumentSpec(attributes);

    // Search for and print any matching instruments.
    vector<Instrument *> matching_instruments = inventory->search(target_spec);
    if (matching_instruments.size() > 0)
    {
        cout << "Erin, you might like these instruments:" << endl << endl;
        vector<Instrument *>::iterator it_instrument;
        for (Instrument *instrument : matching_instruments)
        {
            instrument->print_match();
        }
    }
    else
    {
        cout << "Sorry, we have nothing that matches." << endl;
    }
}

void initialize_inventory(Inventory *inventory)
{
    Attributes *attributes = new Attributes();
    attributes->set(Key::BUILDER,      Builder::COLLINGS);
    attributes->set(Key::MODEL,        string("CJ"));
    attributes->set(Key::TYPE,         Type::ACOUSTIC);
    attributes->set(Key::STRING_COUNT, 6);
    attributes->set(Key::TOP_WOOD,     Wood::INDIAN_ROSEWOOD);
    attributes->set(Key::BACK_WOOD,    Wood::SITKA);
    inventory->add_instrument(Kind::GUITAR, "11277", 3999.95,
                              new InstrumentSpec(attributes));

    attributes = new Attributes();
    attributes->set(Key::BUILDER,      Builder::MARTIN);
    attributes->set(Key::MODEL,        string("D-18"));
    attributes->set(Key::TYPE,         Type::ACOUSTIC);
    attributes->set(Key::STRING_COUNT, 6);
    attributes->set(Key::TOP_WOOD,     Wood::MAHOGANY);
    attributes->set(Key::BACK_WOOD,    Wood::ADIRONDACK);
    inventory->add_instrument(Kind::GUITAR, "122784", 5495.95,
                              new InstrumentSpec(attributes));

    attributes = new Attributes();
    attributes->set(Key::BUILDER,      Builder::FENDER);
    attributes->set(Key::MODEL,        string("Stratocastor"));
    attributes->set(Key::TYPE,         Type::ELECTRIC);
    attributes->set(Key::STRING_COUNT, 6);
    attributes->set(Key::TOP_WOOD,     Wood::ALDER);
    attributes->set(Key::BACK_WOOD,    Wood::ALDER);
    inventory->add_instrument(Kind::GUITAR, "V95693", 1499.95,
                              new InstrumentSpec(attributes));
    inventory->add_instrument(Kind::GUITAR, "V9512", 1549.95,
                              new InstrumentSpec(attributes));

    attributes = new Attributes();
    attributes->set(Key::BUILDER,      Builder::GIBSON);
    attributes->set(Key::MODEL,        string("Les Paul"));
    attributes->set(Key::TYPE,         Type::ELECTRIC);
    attributes->set(Key::STRING_COUNT, 6);
    attributes->set(Key::TOP_WOOD,     Wood::MAPLE);
    attributes->set(Key::BACK_WOOD,    Wood::MAPLE);
    inventory->add_instrument(Kind::GUITAR, "70108276", 2295.95,
                              new InstrumentSpec(attributes));

    attributes = new Attributes();
    attributes->set(Key::BUILDER,      Builder::GIBSON);
    attributes->set(Key::MODEL,        string("SG '61 Reissue"));
    attributes->set(Key::TYPE,         Type::ELECTRIC);
    attributes->set(Key::STRING_COUNT, 6);
    attributes->set(Key::TOP_WOOD,     Wood::MAHOGANY);
    attributes->set(Key::BACK_WOOD,    Wood::MAHOGANY);
    inventory->add_instrument(Kind::GUITAR, "82765501", 1890.95,
                              new InstrumentSpec(attributes));

    attributes = new Attributes();
    attributes->set(Key::BUILDER,   Builder::GIBSON);
    attributes->set(Key::MODEL,     string("F-5G"));
    attributes->set(Key::TYPE,      Type::ACOUSTIC);
    attributes->set(Key::STYLE,     Style::A);
    attributes->set(Key::TOP_WOOD,  Wood::MAPLE);
    attributes->set(Key::BACK_WOOD, Wood::MAPLE);
    inventory->add_instrument(Kind::MANDOLIN, "9019920", 5495.99,
                              new InstrumentSpec(attributes));

    attributes = new Attributes();
    attributes->set(Key::BUILDER,      Builder::GIBSON);
    attributes->set(Key::MODEL,        string("SRB-3 Wreath"));
    attributes->set(Key::TYPE,         Type::ACOUSTIC);
    attributes->set(Key::STRING_COUNT, 5);
    attributes->set(Key::BACK_WOOD,    Wood::MAPLE);
    inventory->add_instrument(Kind::BANJO, "8900231", 2945.95,
                              new InstrumentSpec(attributes));
}
