/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#ifndef MODEL_H_
#define MODEL_H_

#include <string>
#include <boost/any.hpp>

using namespace std;

/**
 * Cast a boost::any value to a string value.
 * @param value the value to cast.
 * @return the string value.
 */
inline string to_model(boost::any value)
{
    return boost::any_cast<string>(value);
}

/**
 * Convert the letters of a string to lower-case.
 * @param str the string to convert.
 * @return the converted string.
 */
inline string lower(string str)
{
    transform(str.begin(), str.end(), str.begin(), ::tolower);
    return str;
}

/**
 * Test whether two Model values are equal.
 * @param value1 the first value.
 * @param value2 the second value.
 * @return true if they're equal, else false.
 */
inline bool equal_models(boost::any value1, boost::any value2)
{
    return lower(to_model(value1)) == lower(to_model(value2));
}

#endif /* MODEL_H_ */
