/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#ifndef TYPE_H_
#define TYPE_H_

#include <iostream>
#include <boost/any.hpp>

using namespace std;

/**
 * The type attribute values.
 */
enum class Type
{
    ACOUSTIC, ELECTRIC
};

/**
 * Cast a boost::any value to a Type value.
 * @param value the value to cast.
 * @return the Type value.
 */
inline Type to_type(boost::any value)
{
    return boost::any_cast<Type>(value);
}

/**
 * Test whether two Type values are equal.
 * @param value1 the first value.
 * @param value2 the second value.
 * @return true if they're equal, else false.
 */
inline bool equal_types(boost::any value1, boost::any value2)
{
    return to_type(value1) == to_type(value2);
}

inline ostream& operator <<(ostream& ostr, const Type type)
{
    switch (type)
    {
        case Type::ACOUSTIC: cout << "acoustic"; break;
        case Type::ELECTRIC: cout << "electric"; break;
        default:             cout << "unspecified";
    }

    return ostr;
}

#endif /* TYPE_H_ */
